function [yui,light] = im_lighting(xui,light,cgamma)

% Adjust lighting of xui using light.
% The 3 elements of light define deviations of a piecewise-linear 
% lighting correction function from unit gradient as follows:
% tx = [64 128 192]'; % Intensity points in xui.
% ty = tx + 256 * light; % Equivalent intensities in yui.
% cgamma defines gamma correction, and should be 1 for no effect.
%
% Nick Kingsbury, Cambridge University, 2006.

wh = waitbar(0,'Relighting image');
sx = size(xui);

% Calculate luminance image.
L1 = 0.3*double(xui(:,:,1)) + 0.6*double(xui(:,:,2)) + 0.1*double(xui(:,:,3)); 

% Blur the luminance image, so as to give a smooth segmentation of xui
% into shadows, mid-tones and highlights.
h = gaussian(4);
waitbar(1/5);

L = colfilter(colfilter(L1,h)',h)';
waitbar(2/5);
% figure(3);image(uint8(L));colormap(gray(256));pause(0.1)

% Define the correction function to give equivalent intensity points 
% in xui and yui.
tx = [64 128 192]'; % Intensity points in xui.
ty = tx + 256 * light; % Equivalent intensities in yui.
ty(2) = max(ty(2),ty(1)+1); % Ensure gradient is always positive.
ty(3) = max(ty(3),ty(2)+1);
if ty(3)>254, error('Inconsistent lighting vector'); end
light = (ty - tx)/256; % Recalculate light to allow for limiting.

% Calculate gradients of the 4 segments of the correction function.
grad = ([ty;255] - [0;ty])./([tx;255] - [0;tx]);

% Find pixels in each segment of the correction function.
s0 = find(L < tx(1));
s1 = find(L >= tx(1) & L < tx(2));
s2 = find(L >= tx(2) & L < tx(3));
s3 = find(L >= tx(3));

% Calculate gain to be applied to each pixel.
gain = ones(sx(1:2));
gain(s0) = grad(1);
gain(s1) = (ty(1) + (L(s1)-tx(1))*grad(2))./L(s1);
gain(s2) = (ty(2) + (L(s2)-tx(2))*grad(3))./L(s2);
gain(s3) = (ty(3) + (L(s3)-tx(3))*grad(4))./L(s3);

% Apply gamma correction to the gains, based on the luminance image L1.
gain = gain .* ((max(L1,1)/255).^(1/cgamma - 1));
% figure(4);image(uint8(gain*64));colormap(gray(256));pause(0.1)

% Perform the relighting using the matrix, gain.
yui = xui;
for k = 1:3,
    waitbar((k+2)/5);
    yui(:,:,k) = uint8(double(xui(:,:,k)) .* gain + 0.5);
end
close(wh) % Close wait bar
return
